package com.testor.module.contractor.manage.controller;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import lombok.extern.slf4j.Slf4j;
import com.tongtech.tfw.backend.core.helper.StringHelper;
import com.tongtech.tfw.backend.core.helper.ObjectHelper;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RestController;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tongtech.tfw.backend.common.biz.constants.BizConstants;
import com.tongtech.tfw.backend.common.biz.models.BaseResponse;
import com.tongtech.tfw.backend.common.biz.models.BaseResponse;
import com.tongtech.tfw.backend.common.biz.models.BaseResponseList;
import com.tongtech.tfw.backend.common.biz.models.BizGeneralResponse;
import com.tongtech.tfw.backend.common.models.exceptions.ApiException;
import com.tongtech.tfw.backend.common.models.supers.SuperController;
import com.tongtech.tfw.backend.core.helper.IdHelper;
import com.tongtech.tfw.backend.core.helper.TypeHelper;
import com.tongtech.tfw.backend.core.helper.bean.BeanHelper;

import com.testor.module.contractor.manage.model.domain.TContractorDailyPlanProtective;
import com.testor.module.contractor.manage.model.dto.TContractorDailyPlanProtectiveParam;
import com.testor.module.contractor.manage.model.dto.TContractorDailyPlanProtectiveDelParam;
import com.testor.module.contractor.manage.service.TContractorDailyPlanProtectiveService;

/**
 * 日常管理计划-防护用品台账关联Controller
 *
 * @author testor-framework
 * @date 2024-12-10 14:08:31
 */
@RestController
@RequestMapping("/manage/protective")
@Api(value = "TContractorDailyPlanProtective", tags = "日常管理计划-防护用品台账关联")
@Slf4j
public class TContractorDailyPlanProtectiveController extends SuperController
{
    @Autowired
    private TContractorDailyPlanProtectiveService tContractorDailyPlanProtectiveService;

    /* Generated Method*/
	@ApiOperation(value = "新增 日常管理计划-防护用品台账关联", notes = "Add TContractorDailyPlanProtective")
	@PostMapping(value = "/add")
	public BaseResponse<TContractorDailyPlanProtective>addEntity(@RequestBody TContractorDailyPlanProtective addRequest) {
		BaseResponse<TContractorDailyPlanProtective> baseResponse=new BaseResponse<>();
		if(ObjectHelper.isNotEmpty(addRequest)){
			// TODO 按需求添加数据去重，特殊值设定，业务异常
			String id = IdHelper.getId32bit();
            TContractorDailyPlanProtective data=BeanHelper.beanToBean(addRequest,  TContractorDailyPlanProtective.class);
			data.setId(id);
			boolean result= tContractorDailyPlanProtectiveService.save(data);
			if(result){
                TContractorDailyPlanProtective newEntity = new TContractorDailyPlanProtective();
				newEntity.setId(id);
				baseResponse.setData(newEntity);
			}
		}
		return baseResponse;
	}

	@ApiOperation(value = "获取 日常管理计划-防护用品台账关联 ", notes = "Get TContractorDailyPlanProtective By Id")
	@GetMapping(value = "/get")
	public BaseResponse<TContractorDailyPlanProtective> getById(String id){
		BaseResponse<TContractorDailyPlanProtective> baseResponse=new BaseResponse<>();
		if(StringHelper.isNotEmpty(id)){
			// TODO 按需求添加业务异常判断
            TContractorDailyPlanProtective data= tContractorDailyPlanProtectiveService.getById(id);
			baseResponse.setData(data);
		}
		return baseResponse;
	}

	@ApiOperation(value = "修改 日常管理计划-防护用品台账关联 ", notes = "Update TContractorDailyPlanProtective By Id")
	@PutMapping(value = "/update")
	public BaseResponse<BizGeneralResponse> updateEntity(@RequestBody TContractorDailyPlanProtective updateRequest){
		BaseResponse<BizGeneralResponse> baseResponse=new BaseResponse<>();
		if(ObjectHelper.isNotEmpty(updateRequest)&&StringHelper.isNotEmpty(updateRequest.getId())){
			// TODO 按需求添加数据去重，特殊值设定，业务异常
            TContractorDailyPlanProtective data=BeanHelper.beanToBean(updateRequest, TContractorDailyPlanProtective.class);
			boolean result= tContractorDailyPlanProtectiveService.updateById(data);
			baseResponse.setData(
					BizGeneralResponse.builder().result(TypeHelper.castToString(result)).build());
		}
		return baseResponse;
	}

	@ApiOperation(value = "删除 日常管理计划-防护用品台账关联 ", notes = "delete TContractorDailyPlanProtective By Id")
	@PutMapping(value = "/delete")
	public BaseResponse<BizGeneralResponse> deleteEntity(@RequestBody TContractorDailyPlanProtectiveDelParam deleteRequest){
		BaseResponse<BizGeneralResponse> baseResponse=new BaseResponse<>();
		if(ObjectHelper.isNotEmpty(deleteRequest)&&StringHelper.isNotEmpty(deleteRequest.getId())){
			// TODO 按需求添加数据去重，特殊值设定，业务异常
            TContractorDailyPlanProtective data= new TContractorDailyPlanProtective();
			data.setId(deleteRequest.getId());
			data.setStatus(BizConstants.STATUS_DELETE);
			boolean result= tContractorDailyPlanProtectiveService.updateById(data);
			baseResponse.setData(
					BizGeneralResponse.builder().result(TypeHelper.castToString(result)).build());
		}
		return baseResponse;
	}

	@ApiOperation(value = "批量删除日常管理计划-防护用品台账关联", notes = "delete TContractorDailyPlanProtective by batch Id")
	@PutMapping(value = "/batch_delete")
	public BaseResponse<BizGeneralResponse> deleteEntityBatch(
			@RequestBody TContractorDailyPlanProtectiveDelParam deleteRequest) {
		BaseResponse<BizGeneralResponse> baseResponse = new BaseResponse<>();
		if (ObjectHelper.isNotEmpty(deleteRequest) && (!deleteRequest.getIds().isEmpty())) {
            /* 批量删除 */
			List<TContractorDailyPlanProtective> tContractorDailyPlanProtectiveList = new ArrayList<>();
            for(String id: deleteRequest.getIds()){
                TContractorDailyPlanProtective data= new TContractorDailyPlanProtective();
				data.setId(id);
				data.setStatus(BizConstants.STATUS_DELETE);
                tContractorDailyPlanProtectiveList.add(data);
			}
			boolean result = tContractorDailyPlanProtectiveService.updateBatchById(tContractorDailyPlanProtectiveList);
			baseResponse.setData(
					BizGeneralResponse.builder().result(TypeHelper.castToString(result)).build());
		}
		return baseResponse;
	}

	@ApiOperation(value = "获取 日常管理计划-防护用品台账关联 分页列表", notes = "List TContractorDailyPlanProtective with page")
	@GetMapping(value = "/list")
	public  BaseResponse<BaseResponseList<TContractorDailyPlanProtective>>listEntity(TContractorDailyPlanProtectiveParam param){
		BaseResponse<BaseResponseList<TContractorDailyPlanProtective>>baseResponse=new BaseResponse<>();
		Long page=
				StringHelper.isEmpty(param.getPage())?BizConstants.PAGE:Long.valueOf(param.getPage());
		Long limit=
				StringHelper.isEmpty(param.getLimit())?BizConstants.LIMIT:Long.valueOf(param.getLimit());
		Page<TContractorDailyPlanProtective> resultPage=new Page<>(page,limit);
		// TODO 根据需求修改查询条件及查询参数
		QueryWrapper<TContractorDailyPlanProtective> queryWrapper=this.createQuery(param);
		Page<TContractorDailyPlanProtective> resultList= tContractorDailyPlanProtectiveService.page(resultPage,queryWrapper);
		BaseResponseList<TContractorDailyPlanProtective> baseResponseList=new BaseResponseList<>();
		baseResponseList.setData(resultList.getRecords());
		baseResponseList.setTotal(resultList.getTotal());
		baseResponse.setData(baseResponseList);
		return baseResponse;
	}

	@ApiOperation(value = "获取 日常管理计划-防护用品台账关联 列表", notes = "List TContractorDailyPlanProtective all")
	@GetMapping(value = "/datas")
	public  BaseResponse<BaseResponseList<TContractorDailyPlanProtective>>listAllEntity(TContractorDailyPlanProtectiveParam param){
		BaseResponse<BaseResponseList<TContractorDailyPlanProtective>>baseResponse=new BaseResponse<>();
		// TODO 根据需求修改查询条件及查询参数
		QueryWrapper<TContractorDailyPlanProtective> queryWrapper=this.createQuery(param);
		List<TContractorDailyPlanProtective> resultList=tContractorDailyPlanProtectiveService.list(queryWrapper);
		BaseResponseList<TContractorDailyPlanProtective> baseResponseList=new BaseResponseList<>();
		baseResponseList.setData(resultList);
		baseResponseList.setTotal(resultList.size());
		baseResponse.setData(baseResponseList);
		return baseResponse;
	}

    /* Private Methods */
	/**
     * 列表查询条件及查询参数
     */
	private QueryWrapper<TContractorDailyPlanProtective> createQuery(TContractorDailyPlanProtectiveParam queryParam){
		QueryWrapper<TContractorDailyPlanProtective> queryWrapper=new QueryWrapper<>();
		if(StringHelper.isNotEmpty(queryParam.getPlanId())){
			queryWrapper.eq(TContractorDailyPlanProtective.PLAN_ID,queryParam.getPlanId());
		}
		if(StringHelper.isNotEmpty(queryParam.getProtectiveId())){
			queryWrapper.eq(TContractorDailyPlanProtective.PROTECTIVE_ID,queryParam.getProtectiveId());
		}
		if(StringHelper.isNotEmpty(queryParam.getBizId())){
			queryWrapper.eq(TContractorDailyPlanProtective.BIZ_ID,queryParam.getBizId());
		}
		if(StringHelper.isNotEmpty(queryParam.getContractorId())){
			queryWrapper.eq(TContractorDailyPlanProtective.CONTRACTOR_ID,queryParam.getContractorId());
		}
		if(StringHelper.isNotEmpty(queryParam.getProjId())){
			queryWrapper.eq(TContractorDailyPlanProtective.PROJ_ID,queryParam.getProjId());
		}
		if (StringHelper.isNotEmpty(queryParam.getStatus())) {
			queryWrapper.eq(TContractorDailyPlanProtective.STATUS, queryParam.getStatus());
		} else {
			queryWrapper.ne(TContractorDailyPlanProtective.STATUS, BizConstants.STATUS_DELETE);
		}
		if (StringHelper.isNotEmpty(queryParam.getOrderBy())){
			if(StringHelper.isNotEmpty(queryParam.getOrderType())
					&&BizConstants.ASC.equals(queryParam.getOrderType())){
				queryWrapper.orderByAsc(queryParam.getOrderBy());
			}else{
				queryWrapper.orderByDesc(queryParam.getOrderBy());
			}
		}else{
			queryWrapper.orderByDesc(TContractorDailyPlanProtective.UPDATE_DATE);
		}
		return queryWrapper;
	}
}
